@@ -5,7 +5,7 @@
 
     Several HTML builders.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -14,29 +14,25 @@ import sys
 import zlib
 import codecs
 import posixpath
-import cPickle as pickle
 from os import path
-try:
-    from hashlib import md5
-except ImportError:
-    # 2.4 compatibility
-    from md5 import md5
+from hashlib import md5
 
+from six import iteritems, text_type, string_types
+from six.moves import cPickle as pickle
 from docutils import nodes
 from docutils.io import DocTreeInput, StringOutput
 from docutils.core import Publisher
-from docutils.utils import new_document
+from docutils.utils import new_document, relative_path
 from docutils.frontend import OptionParser
 from docutils.readers.doctree import Reader as DoctreeReader
 
-from sphinx import package_dir, __version__
-from sphinx.util import jsonimpl, copy_static_entry
+from sphinx import package_dir, __display_version__
+from sphinx.util import jsonimpl, copy_static_entry, copy_extra_entry
+from sphinx.util.i18n import format_date
 from sphinx.util.osutil import SEP, os_path, relative_uri, ensuredir, \
-     movefile, ustrftime, copyfile
+    movefile, copyfile
 from sphinx.util.nodes import inline_all_toctrees
 from sphinx.util.matching import patmatch, compile_matchers
-from sphinx.util.pycompat import any, b
-from sphinx.errors import SphinxError
 from sphinx.locale import _
 from sphinx.search import js_index
 from sphinx.theming import Theme
@@ -45,7 +41,7 @@ from sphinx.application import ENV_PICKLE_FILENAME
 from sphinx.highlighting import PygmentsBridge
 from sphinx.util.console import bold, darkgreen, brown
 from sphinx.writers.html import HTMLWriter, HTMLTranslator, \
-     SmartyPantsHTMLTranslator
+    SmartyPantsHTMLTranslator
 
 #: the filename for the inventory of objects
 INVENTORY_FILENAME = 'objects.inv'
@@ -53,6 +49,19 @@ INVENTORY_FILENAME = 'objects.inv'
 LAST_BUILD_FILENAME = 'last_build'
 
 
+def get_stable_hash(obj):
+    """
+    Return a stable hash for a Python data structure.  We can't just use
+    the md5 of str(obj) since for example dictionary items are enumerated
+    in unpredictable order due to hash randomization in newer Pythons.
+    """
+    if isinstance(obj, dict):
+        return get_stable_hash(list(obj.items()))
+    elif isinstance(obj, (list, tuple)):
+        obj = sorted(get_stable_hash(o) for o in obj)
+    return md5(text_type(obj).encode('utf8')).hexdigest()
+
+
 class StandaloneHTMLBuilder(Builder):
     """
     Builds standalone HTML docs.
@@ -60,6 +69,7 @@ class StandaloneHTMLBuilder(Builder):
     name = 'html'
     format = 'html'
     copysource = True
+    allow_parallel = True
     out_suffix = '.html'
     link_suffix = '.html'  # defaults to matching out_suffix
     indexer_format = js_index
@@ -69,6 +79,7 @@ class StandaloneHTMLBuilder(Builder):
     searchindex_filename = 'searchindex.js'
     add_permalinks = True
     embedded = False  # for things like HTML help or Qt help: suppresses sidebar
+    search = True  # for things like HTML help and Apple help: suppress search
 
     # This is a class attribute because it is mutated by Sphinx.add_javascript.
     script_files = ['_static/jquery.js', '_static/underscore.js',
@@ -86,6 +97,8 @@ class StandaloneHTMLBuilder(Builder):
         # a hash of all config values that, if changed, cause a full rebuild
         self.config_hash = ''
         self.tags_hash = ''
+        # basename of images directory
+        self.imagedir = '_images'
         # section numbers for headings in the currently visited document
         self.secnumbers = {}
         # currently written docname
@@ -126,7 +139,7 @@ class StandaloneHTMLBuilder(Builder):
         Theme.init_themes(self.confdir, self.config.html_theme_path,
                           warn=self.warn)
         themename, themeoptions = self.get_theme_config()
-        self.theme = Theme(themename)
+        self.theme = Theme(themename, warn=self.warn)
         self.theme_options = themeoptions.copy()
         self.create_template_bridge()
         self.templates.init(self, self.theme)
@@ -143,7 +156,9 @@ class StandaloneHTMLBuilder(Builder):
                                           self.config.trim_doctest_flags)
 
     def init_translator_class(self):
-        if self.config.html_translator_class:
+        if self.translator_class is not None:
+            pass
+        elif self.config.html_translator_class:
             self.translator_class = self.app.import_object(
                 self.config.html_translator_class,
                 'html_translator_class setting')
@@ -154,11 +169,10 @@ class StandaloneHTMLBuilder(Builder):
 
     def get_outdated_docs(self):
         cfgdict = dict((name, self.config[name])
-                       for (name, desc) in self.config.values.iteritems()
+                       for (name, desc) in iteritems(self.config.values)
                        if desc[1] == 'html')
-        self.config_hash = md5(unicode(cfgdict).encode('utf-8')).hexdigest()
-        self.tags_hash = md5(unicode(sorted(self.tags)).encode('utf-8')) \
-                .hexdigest()
+        self.config_hash = get_stable_hash(cfgdict)
+        self.tags_hash = get_stable_hash(sorted(self.tags))
         old_config_hash = old_tags_hash = ''
         try:
             fp = open(path.join(self.outdir, '.buildinfo'))
@@ -181,7 +195,7 @@ class StandaloneHTMLBuilder(Builder):
         except Exception:
             pass
         if old_config_hash != self.config_hash or \
-               old_tags_hash != self.tags_hash:
+           old_tags_hash != self.tags_hash:
             for docname in self.env.found_docs:
                 yield docname
             return
@@ -212,13 +226,13 @@ class StandaloneHTMLBuilder(Builder):
         """Utility: Render a lone doctree node."""
         if node is None:
             return {'fragment': ''}
-        doc = new_document(b('<partial node>'))
+        doc = new_document(b'<partial node>')
         doc.append(node)
 
         if self._publisher is None:
             self._publisher = Publisher(
-                    source_class = DocTreeInput,
-                    destination_class=StringOutput)
+                source_class = DocTreeInput,
+                destination_class=StringOutput)
             self._publisher.set_components('standalone',
                                            'restructuredtext', 'pseudoxml')
 
@@ -235,18 +249,22 @@ class StandaloneHTMLBuilder(Builder):
 
     def prepare_writing(self, docnames):
         # create the search indexer
-        from sphinx.search import IndexBuilder, languages
-        lang = self.config.html_search_language or self.config.language
-        if not lang or lang not in languages:
-            lang = 'en'
-        self.indexer = IndexBuilder(self.env, lang,
-                                    self.config.html_search_options)
-        self.load_indexer(docnames)
+        self.indexer = None
+        if self.search:
+            from sphinx.search import IndexBuilder, languages
+            lang = self.config.html_search_language or self.config.language
+            if not lang or lang not in languages:
+                lang = 'en'
+            self.indexer = IndexBuilder(self.env, lang,
+                                        self.config.html_search_options,
+                                        self.config.html_search_scorer)
+            self.load_indexer(docnames)
 
         self.docwriter = HTMLWriter(self)
         self.docsettings = OptionParser(
             defaults=self.env.settings,
-            components=(self.docwriter,)).get_default_values()
+            components=(self.docwriter,),
+            read_config_files=True).get_default_values()
         self.docsettings.compact_lists = bool(self.config.html_compact_lists)
 
         # determine the additional indices to include
@@ -254,7 +272,8 @@ class StandaloneHTMLBuilder(Builder):
         # html_domain_indices can be False/True or a list of index names
         indices_config = self.config.html_domain_indices
         if indices_config:
-            for domain in self.env.domains.itervalues():
+            for domain_name in sorted(self.env.domains):
+                domain = self.env.domains[domain_name]
                 for indexcls in domain.indices:
                     indexname = '%s-%s' % (domain.name, indexcls.name)
                     if isinstance(indices_config, list):
@@ -262,7 +281,7 @@ class StandaloneHTMLBuilder(Builder):
                             continue
                     # deprecated config value
                     if indexname == 'py-modindex' and \
-                           not self.config.html_use_modindex:
+                       not self.config.html_use_modindex:
                         continue
                     content, collapse = indexcls(domain).generate()
                     if content:
@@ -273,25 +292,26 @@ class StandaloneHTMLBuilder(Builder):
         # typically doesn't include the time of day
         lufmt = self.config.html_last_updated_fmt
         if lufmt is not None:
-            self.last_updated = ustrftime(lufmt or _('%b %d, %Y'))
+            self.last_updated = format_date(lufmt or _('MMM dd, YYYY'),
+                                            language=self.config.language)
         else:
             self.last_updated = None
 
         logo = self.config.html_logo and \
-               path.basename(self.config.html_logo) or ''
+            path.basename(self.config.html_logo) or ''
 
         favicon = self.config.html_favicon and \
-                  path.basename(self.config.html_favicon) or ''
+            path.basename(self.config.html_favicon) or ''
         if favicon and os.path.splitext(favicon)[1] != '.ico':
             self.warn('html_favicon is not an .ico file')
 
-        if not isinstance(self.config.html_use_opensearch, basestring):
+        if not isinstance(self.config.html_use_opensearch, string_types):
             self.warn('html_use_opensearch config value must now be a string')
 
         self.relations = self.env.collect_relations()
 
         rellinks = []
-        if self.config.html_use_index:
+        if self.get_builder_config('use_index', 'html'):
             rellinks.append(('genindex', _('General Index'), 'I', _('index')))
         for indexname, indexcls, content, collapse in self.domain_indices:
             # if it has a short name
@@ -323,8 +343,9 @@ class StandaloneHTMLBuilder(Builder):
             show_source = self.config.html_show_sourcelink,
             file_suffix = self.out_suffix,
             script_files = self.script_files,
+            language = self.config.language,
             css_files = self.css_files,
-            sphinx_version = __version__,
+            sphinx_version = __display_version__,
             style = stylename,
             rellinks = rellinks,
             builder = self.name,
@@ -335,7 +356,7 @@ class StandaloneHTMLBuilder(Builder):
         if self.theme:
             self.globalcontext.update(
                 ('theme_' + key, val) for (key, val) in
-                self.theme.get_options(self.theme_options).iteritems())
+                iteritems(self.theme.get_options(self.theme_options)))
         self.globalcontext.update(self.config.html_context)
 
     def get_doc_context(self, docname, body, metatags):
@@ -375,8 +396,9 @@ class StandaloneHTMLBuilder(Builder):
                 pass
             related = self.relations.get(related[0])
         if parents:
-            parents.pop() # remove link to the master file; we have a generic
-                          # "back to index" link already
+            # remove link to the master file; we have a generic
+            # "back to index" link already
+            parents.pop()
         parents.reverse()
 
         # title rendered as HTML
@@ -388,6 +410,9 @@ class StandaloneHTMLBuilder(Builder):
         # metadata for the document
         meta = self.env.metadata.get(docname)
 
+        # Suffix for the document
+        source_suffix = '.' + self.env.doc2path(docname).split('.')[-1]
+
         # local TOC and global TOC tree
         self_toc = self.env.get_toc_for(docname, self)
         toc = self.render_partial(self_toc)['fragment']
@@ -405,6 +430,7 @@ class StandaloneHTMLBuilder(Builder):
             toc = toc,
             # only display a TOC if there's more than one item to show
             display_toc = (self.env.toc_num_entries[docname] > 1),
+            page_source_suffix = source_suffix,
         )
 
     def write_doc(self, docname, doctree):
@@ -412,8 +438,8 @@ class StandaloneHTMLBuilder(Builder):
         doctree.settings = self.docsettings
 
         self.secnumbers = self.env.toc_secnumbers.get(docname, {})
+        self.fignumbers = self.env.toc_fignumbers.get(docname, {})
         self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
-        self.post_process_images(doctree)
         self.dlpath = relative_uri(self.get_target_uri(docname), '_downloads')
         self.current_docname = docname
         self.docwriter.write(doctree, destination)
@@ -422,57 +448,73 @@ class StandaloneHTMLBuilder(Builder):
         metatags = self.docwriter.clean_meta
 
         ctx = self.get_doc_context(docname, body, metatags)
-        self.index_page(docname, doctree, ctx.get('title', ''))
         self.handle_page(docname, ctx, event_arg=doctree)
 
-    def finish(self):
-        self.info(bold('writing additional files...'), nonl=1)
+    def write_doc_serialized(self, docname, doctree):
+        self.imgpath = relative_uri(self.get_target_uri(docname), self.imagedir)
+        self.post_process_images(doctree)
+        title = self.env.longtitles.get(docname)
+        title = title and self.render_partial(title)['title'] or ''
+        self.index_page(docname, doctree, title)
 
-        # pages from extensions
-        for pagelist in self.app.emit('html-collect-pages'):
-            for pagename, context, template in pagelist:
-                self.handle_page(pagename, context, template)
+    def finish(self):
+        self.finish_tasks.add_task(self.gen_indices)
+        self.finish_tasks.add_task(self.gen_additional_pages)
+        self.finish_tasks.add_task(self.copy_image_files)
+        self.finish_tasks.add_task(self.copy_download_files)
+        self.finish_tasks.add_task(self.copy_static_files)
+        self.finish_tasks.add_task(self.copy_extra_files)
+        self.finish_tasks.add_task(self.write_buildinfo)
+
+        # dump the search index
+        self.handle_finish()
+
+    def gen_indices(self):
+        self.info(bold('generating indices...'), nonl=1)
 
         # the global general index
-        if self.config.html_use_index:
+        if self.get_builder_config('use_index', 'html'):
             self.write_genindex()
 
         # the global domain-specific indices
         self.write_domain_indices()
 
-        # the search page
-        if self.name != 'htmlhelp':
-            self.info(' search', nonl=1)
-            self.handle_page('search', {}, 'search.html')
+        self.info()
+
+    def gen_additional_pages(self):
+        # pages from extensions
+        for pagelist in self.app.emit('html-collect-pages'):
+            for pagename, context, template in pagelist:
+                self.handle_page(pagename, context, template)
+
+        self.info(bold('writing additional pages...'), nonl=1)
 
         # additional pages from conf.py
         for pagename, template in self.config.html_additional_pages.items():
             self.info(' '+pagename, nonl=1)
             self.handle_page(pagename, {}, template)
 
-        if self.config.html_use_opensearch and self.name != 'htmlhelp':
+        # the search page
+        if self.search:
+            self.info(' search', nonl=1)
+            self.handle_page('search', {}, 'search.html')
+
+        # the opensearch xml file
+        if self.config.html_use_opensearch and self.search:
             self.info(' opensearch', nonl=1)
             fn = path.join(self.outdir, '_static', 'opensearch.xml')
             self.handle_page('opensearch', {}, 'opensearch.xml', outfilename=fn)
 
         self.info()
 
-        self.copy_image_files()
-        self.copy_download_files()
-        self.copy_static_files()
-        self.write_buildinfo()
-
-        # dump the search index
-        self.handle_finish()
-
     def write_genindex(self):
         # the total count of lines for each index letter, used to distribute
         # the entries into two columns
         genindex = self.env.create_index(self)
         indexcounts = []
-        for _, entries in genindex:
+        for _k, entries in genindex:
             indexcounts.append(sum(1 + len(subitems)
-                                   for _, (_, subitems) in entries))
+                                   for _, (_, subitems, _) in entries))
 
         genindexcontext = dict(
             genindexentries = genindex,
@@ -507,29 +549,32 @@ class StandaloneHTMLBuilder(Builder):
     def copy_image_files(self):
         # copy image files
         if self.images:
-            ensuredir(path.join(self.outdir, '_images'))
-            for src in self.status_iterator(self.images, 'copying images... ',
-                                            brown, len(self.images)):
+            ensuredir(path.join(self.outdir, self.imagedir))
+            for src in self.app.status_iterator(self.images, 'copying images... ',
+                                                brown, len(self.images)):
                 dest = self.images[src]
                 try:
                     copyfile(path.join(self.srcdir, src),
-                             path.join(self.outdir, '_images', dest))
-                except Exception, err:
+                             path.join(self.outdir, self.imagedir, dest))
+                except Exception as err:
                     self.warn('cannot copy image file %r: %s' %
                               (path.join(self.srcdir, src), err))
 
     def copy_download_files(self):
+        def to_relpath(f):
+            return relative_path(self.srcdir, f)
         # copy downloadable files
         if self.env.dlfiles:
             ensuredir(path.join(self.outdir, '_downloads'))
-            for src in self.status_iterator(self.env.dlfiles,
-                                            'copying downloadable files... ',
-                                            brown, len(self.env.dlfiles)):
+            for src in self.app.status_iterator(self.env.dlfiles,
+                                                'copying downloadable files... ',
+                                                brown, len(self.env.dlfiles),
+                                                stringify_func=to_relpath):
                 dest = self.env.dlfiles[src][1]
                 try:
                     copyfile(path.join(self.srcdir, src),
                              path.join(self.outdir, '_downloads', dest))
-                except Exception, err:
+                except Exception as err:
                     self.warn('cannot copy downloadable file %r: %s' %
                               (path.join(self.srcdir, src), err))
 
@@ -548,9 +593,17 @@ class StandaloneHTMLBuilder(Builder):
                 copyfile(jsfile, path.join(self.outdir, '_static',
                                            'translations.js'))
 
-        # add context items for search function used in searchtools.js_t
+        # copy non-minified stemmer JavaScript file
+        if self.indexer is not None:
+            jsfile = self.indexer.get_js_stemmer_rawcode()
+            if jsfile:
+                copyfile(jsfile, path.join(self.outdir, '_static', '_stemmer.js'))
+
         ctx = self.globalcontext.copy()
-        ctx.update(self.indexer.context_for_searchtool())
+
+        # add context items for search function used in searchtools.js_t
+        if self.indexer is not None:
+            ctx.update(self.indexer.context_for_searchtool())
 
         # then, copy over theme-supplied static files
         if self.theme:
@@ -562,10 +615,7 @@ class StandaloneHTMLBuilder(Builder):
         # then, copy over all user-supplied static files
         staticentries = [path.join(self.confdir, spath)
                          for spath in self.config.html_static_path]
-        matchers = compile_matchers(
-            self.config.exclude_patterns +
-            ['**/' + d for d in self.config.exclude_dirnames]
-        )
+        matchers = compile_matchers(self.config.exclude_patterns)
         for entry in staticentries:
             if not path.exists(entry):
                 self.warn('html_static_path entry %r does not exist' % entry)
@@ -576,17 +626,34 @@ class StandaloneHTMLBuilder(Builder):
         if self.config.html_logo:
             logobase = path.basename(self.config.html_logo)
             logotarget = path.join(self.outdir, '_static', logobase)
-            if not path.isfile(logotarget):
+            if not path.isfile(path.join(self.confdir, self.config.html_logo)):
+                self.warn('logo file %r does not exist' % self.config.html_logo)
+            elif not path.isfile(logotarget):
                 copyfile(path.join(self.confdir, self.config.html_logo),
                          logotarget)
         if self.config.html_favicon:
             iconbase = path.basename(self.config.html_favicon)
             icontarget = path.join(self.outdir, '_static', iconbase)
-            if not path.isfile(icontarget):
+            if not path.isfile(path.join(self.confdir, self.config.html_favicon)):
+                self.warn('favicon file %r does not exist' % self.config.html_favicon)
+            elif not path.isfile(icontarget):
                 copyfile(path.join(self.confdir, self.config.html_favicon),
                          icontarget)
         self.info('done')
 
+    def copy_extra_files(self):
+        # copy html_extra_path files
+        self.info(bold('copying extra files... '), nonl=True)
+        extraentries = [path.join(self.confdir, epath)
+                        for epath in self.config.html_extra_path]
+        matchers = compile_matchers(self.config.exclude_patterns)
+        for entry in extraentries:
+            if not path.exists(entry):
+                self.warn('html_extra_path entry %r does not exist' % entry)
+                continue
+            copy_extra_entry(entry, self.outdir, matchers)
+        self.info('done')
+
     def write_buildinfo(self):
         # write build info file
         fp = open(path.join(self.outdir, '.buildinfo'), 'w')
@@ -609,22 +676,25 @@ class StandaloneHTMLBuilder(Builder):
         their high res version.
         """
         Builder.post_process_images(self, doctree)
-        for node in doctree.traverse(nodes.image):
-            if not node.has_key('scale') or \
-               isinstance(node.parent, nodes.reference):
-                # docutils does unfortunately not preserve the
-                # ``target`` attribute on images, so we need to check
-                # the parent node here.
-                continue
-            uri = node['uri']
-            reference = nodes.reference('', '', internal=True)
-            if uri in self.images:
-                reference['refuri'] = posixpath.join(self.imgpath,
-                                                     self.images[uri])
-            else:
-                reference['refuri'] = uri
-            node.replace_self(reference)
-            reference.append(node)
+
+        if self.config.html_scaled_image_link:
+            for node in doctree.traverse(nodes.image):
+                scale_keys = ('scale', 'width', 'height')
+                if not any((key in node) for key in scale_keys) or \
+                   isinstance(node.parent, nodes.reference):
+                    # docutils does unfortunately not preserve the
+                    # ``target`` attribute on images, so we need to check
+                    # the parent node here.
+                    continue
+                uri = node['uri']
+                reference = nodes.reference('', '', internal=True)
+                if uri in self.images:
+                    reference['refuri'] = posixpath.join(self.imgpath,
+                                                         self.images[uri])
+                else:
+                    reference['refuri'] = uri
+                node.replace_self(reference)
+                reference.append(node)
 
     def load_indexer(self, docnames):
         keep = set(self.env.all_docs) - set(docnames)
@@ -652,6 +722,8 @@ class StandaloneHTMLBuilder(Builder):
             self.indexer.feed(pagename, title, doctree)
 
     def _get_local_toctree(self, docname, collapse=True, **kwds):
+        if 'includehidden' not in kwds:
+            kwds['includehidden'] = False
         return self.render_partial(self.env.get_toctree_for(
             docname, self, collapse, **kwds))['fragment']
 
@@ -664,7 +736,7 @@ class StandaloneHTMLBuilder(Builder):
         sidebars = None
         matched = None
         customsidebar = None
-        for pattern, patsidebars in self.config.html_sidebars.iteritems():
+        for pattern, patsidebars in iteritems(self.config.html_sidebars):
             if patmatch(pagename, pattern):
                 if matched:
                     if has_wildcard(pattern):
@@ -681,7 +753,7 @@ class StandaloneHTMLBuilder(Builder):
         if sidebars is None:
             # keep defaults
             pass
-        elif isinstance(sidebars, basestring):
+        elif isinstance(sidebars, string_types):
             # 0.x compatible mode: insert custom sidebar before searchbox
             customsidebar = sidebars
             sidebars = None
@@ -721,8 +793,10 @@ class StandaloneHTMLBuilder(Builder):
         self.add_sidebars(pagename, ctx)
         ctx.update(addctx)
 
-        self.app.emit('html-page-context', pagename, templatename,
-                      ctx, event_arg)
+        newtmpl = self.app.emit_firstresult('html-page-context', pagename,
+                                            templatename, ctx, event_arg)
+        if newtmpl:
+            templatename = newtmpl
 
         try:
             output = self.templates.render(templatename, ctx)
@@ -742,7 +816,7 @@ class StandaloneHTMLBuilder(Builder):
                 f.write(output)
             finally:
                 f.close()
-        except (IOError, OSError), err:
+        except (IOError, OSError) as err:
             self.warn("error writing file %s: %s" % (outfilename, err))
         if self.copysource and ctx.get('sourcename'):
             # copy the source file for the "show source" link
@@ -752,8 +826,9 @@ class StandaloneHTMLBuilder(Builder):
             copyfile(self.env.doc2path(pagename), source_name)
 
     def handle_finish(self):
-        self.dump_search_index()
-        self.dump_inventory()
+        if self.indexer:
+            self.finish_tasks.add_task(self.dump_search_index)
+        self.finish_tasks.add_task(self.dump_inventory)
 
     def dump_inventory(self):
         self.info(bold('dumping object inventory... '), nonl=True)
@@ -763,29 +838,31 @@ class StandaloneHTMLBuilder(Builder):
                      u'# Project: %s\n'
                      u'# Version: %s\n'
                      u'# The remainder of this file is compressed using zlib.\n'
-                     % (self.config.project, self.config.version)
-                    ).encode('utf-8'))
+                     % (self.config.project, self.config.version)).encode('utf-8'))
             compressor = zlib.compressobj(9)
-            for domainname, domain in self.env.domains.iteritems():
+            for domainname, domain in sorted(self.env.domains.items()):
                 for name, dispname, type, docname, anchor, prio in \
-                        domain.get_objects():
+                        sorted(domain.get_objects()):
                     if anchor.endswith(name):
                         # this can shorten the inventory by as much as 25%
                         anchor = anchor[:-len(name)] + '$'
-                    uri = self.get_target_uri(docname) + '#' + anchor
+                    uri = self.get_target_uri(docname)
+                    if anchor:
+                        uri += '#' + anchor
                     if dispname == name:
                         dispname = u'-'
                     f.write(compressor.compress(
                         (u'%s %s:%s %s %s %s\n' % (name, domainname, type,
-                                                   prio, uri, dispname)
-                        ).encode('utf-8')))
+                                                   prio, uri, dispname)).encode('utf-8')))
             f.write(compressor.flush())
         finally:
             f.close()
         self.info('done')
 
     def dump_search_index(self):
-        self.info(bold('dumping search index... '), nonl=True)
+        self.info(
+            bold('dumping search index in %s ... ' % self.indexer.label()),
+            nonl=True)
         self.indexer.prune(self.env.all_docs)
         searchindexfn = path.join(self.outdir, self.searchindex_filename)
         # first write to a temporary file, so that if dumping fails,
@@ -814,13 +891,13 @@ class DirectoryHTMLBuilder(StandaloneHTMLBuilder):
         if docname == 'index':
             return ''
         if docname.endswith(SEP + 'index'):
-            return docname[:-5] # up to sep
+            return docname[:-5]  # up to sep
         return docname + SEP
 
     def get_outfilename(self, pagename):
         if pagename == 'index' or pagename.endswith(SEP + 'index'):
-            outfilename = path.join(self.outdir, os_path(pagename)
-                                    + self.out_suffix)
+            outfilename = path.join(self.outdir, os_path(pagename) +
+                                    self.out_suffix)
         else:
             outfilename = path.join(self.outdir, os_path(pagename),
                                     'index' + self.out_suffix)
@@ -847,7 +924,7 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
         if docname in self.env.all_docs:
             # all references are on the same page...
             return self.config.master_doc + self.out_suffix + \
-                   '#document-' + docname
+                '#document-' + docname
         else:
             # chances are this is a html_additional_page
             return docname + self.out_suffix
@@ -870,15 +947,39 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
             if hashindex >= 0:
                 refnode['refuri'] = fname + refuri[hashindex:]
 
+    def _get_local_toctree(self, docname, collapse=True, **kwds):
+        if 'includehidden' not in kwds:
+            kwds['includehidden'] = False
+        toctree = self.env.get_toctree_for(docname, self, collapse, **kwds)
+        self.fix_refuris(toctree)
+        return self.render_partial(toctree)['fragment']
+
     def assemble_doctree(self):
         master = self.config.master_doc
         tree = self.env.get_doctree(master)
-        tree = inline_all_toctrees(self, set(), master, tree, darkgreen)
+        tree = inline_all_toctrees(self, set(), master, tree, darkgreen, [master])
         tree['docname'] = master
         self.env.resolve_references(tree, master, self)
         self.fix_refuris(tree)
         return tree
 
+    def assemble_toc_secnumbers(self):
+        # Assemble toc_secnumbers to resolve section numbers on SingleHTML.
+        # Merge all secnumbers to single secnumber.
+        #
+        # Note: current Sphinx has refid confliction in singlehtml mode.
+        #       To avoid the problem, it replaces key of secnumbers to
+        #       tuple of docname and refid.
+        #
+        #       There are related codes in inline_all_toctres() and
+        #       HTMLTranslter#add_secnumber().
+        new_secnumbers = {}
+        for docname, secnums in iteritems(self.env.toc_secnumbers):
+            for id, secnum in iteritems(secnums):
+                new_secnumbers[(docname, id)] = secnum
+
+        return {self.config.master_doc: new_secnumbers}
+
     def get_doc_context(self, docname, body, metatags):
         # no relation links...
         toc = self.env.get_toctree_for(self.config.master_doc, self, False)
@@ -914,8 +1015,10 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
 
         self.info(bold('assembling single document... '), nonl=True)
         doctree = self.assemble_doctree()
+        self.env.toc_secnumbers = self.assemble_toc_secnumbers()
         self.info()
         self.info(bold('writing... '), nonl=True)
+        self.write_doc_serialized(self.config.master_doc, doctree)
         self.write_doc(self.config.master_doc, doctree)
         self.info('done')
 
@@ -938,6 +1041,7 @@ class SingleFileHTMLBuilder(StandaloneHTMLBuilder):
         self.copy_image_files()
         self.copy_download_files()
         self.copy_static_files()
+        self.copy_extra_files()
         self.write_buildinfo()
         self.dump_inventory()
 
@@ -963,6 +1067,7 @@ class SerializingHTMLBuilder(StandaloneHTMLBuilder):
     def init(self):
         self.config_hash = ''
         self.tags_hash = ''
+        self.imagedir = '_images'
         self.theme = None       # no theme necessary
         self.templates = None   # no template bridge necessary
         self.init_translator_class()
@@ -972,7 +1077,7 @@ class SerializingHTMLBuilder(StandaloneHTMLBuilder):
         if docname == 'index':
             return ''
         if docname.endswith(SEP + 'index'):
-            return docname[:-5] # up to sep
+            return docname[:-5]  # up to sep
         return docname + SEP
 
     def dump_context(self, context, filename):
@@ -994,8 +1099,9 @@ class SerializingHTMLBuilder(StandaloneHTMLBuilder):
             outfilename = path.join(self.outdir,
                                     os_path(pagename) + self.out_suffix)
 
-        self.app.emit('html-page-context', pagename, templatename,
-                      ctx, event_arg)
+        # we're not taking the return value here, since no template is
+        # actually rendered
+        self.app.emit('html-page-context', pagename, templatename, ctx, event_arg)
 
         ensuredir(path.dirname(outfilename))
         self.dump_context(ctx, outfilename)
@@ -1058,8 +1164,4 @@ class JSONHTMLBuilder(SerializingHTMLBuilder):
     searchindex_filename = 'searchindex.json'
 
     def init(self):
-        if jsonimpl.json is None:
-            raise SphinxError(
-                'The module simplejson (or json in Python >= 2.6) '
-                'is not available. The JSONHTMLBuilder builder will not work.')
         SerializingHTMLBuilder.init(self)
