@@ -5,15 +5,16 @@
 
     Glue code for the jinja2 templating engine.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
 from pprint import pformat
 
+from six import string_types
 from jinja2 import FileSystemLoader, BaseLoader, TemplateNotFound, \
-     contextfunction
+    contextfunction
 from jinja2.utils import open_if_exists
 from jinja2.sandbox import SandboxedEnvironment
 
@@ -22,16 +23,18 @@ from sphinx.util.osutil import mtimes_of_files
 
 
 def _tobool(val):
-    if isinstance(val, basestring):
+    if isinstance(val, string_types):
         return val.lower() in ('true', '1', 'yes', 'on')
     return bool(val)
 
+
 def _toint(val):
     try:
         return int(val)
     except ValueError:
         return 0
 
+
 def accesskey(context, key):
     """Helper to output each access key only once."""
     if '_accesskeys' not in context:
@@ -41,11 +44,14 @@ def accesskey(context, key):
         return 'accesskey="%s"' % key
     return ''
 
+
 class idgen(object):
     def __init__(self):
         self.id = 0
+
     def current(self):
         return self.id
+
     def __next__(self):
         self.id += 1
         return self.id
@@ -70,6 +76,7 @@ class SphinxFileSystemLoader(FileSystemLoader):
                 f.close()
 
             mtime = path.getmtime(filename)
+
             def uptodate():
                 try:
                     return path.getmtime(filename) == mtime
@@ -79,7 +86,6 @@ class SphinxFileSystemLoader(FileSystemLoader):
         raise TemplateNotFound(template)
 
 
-
 class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
     """
     Interfaces the rendering environment of jinja2 for use in Sphinx.
@@ -91,25 +97,29 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
         # create a chain of paths to search
         if theme:
             # the theme's own dir and its bases' dirs
-            chain = theme.get_dirchain()
+            pathchain = theme.get_dirchain()
             # then the theme parent paths
-            chain.extend(theme.themepath)
+            loaderchain = pathchain + theme.themepath
         elif dirs:
-            chain = list(dirs)
+            pathchain = list(dirs)
+            loaderchain = list(dirs)
         else:
-            chain = []
+            pathchain = []
+            loaderchain = []
 
         # prepend explicit template paths
         self.templatepathlen = len(builder.config.templates_path)
         if builder.config.templates_path:
-            chain[0:0] = [path.join(builder.confdir, tp)
-                          for tp in builder.config.templates_path]
+            cfg_templates_path = [path.join(builder.confdir, tp)
+                                  for tp in builder.config.templates_path]
+            pathchain[0:0] = cfg_templates_path
+            loaderchain[0:0] = cfg_templates_path
 
         # store it for use in newest_template_mtime
-        self.pathchain = chain
+        self.pathchain = pathchain
 
         # make the paths into loaders
-        self.loaders = map(SphinxFileSystemLoader, chain)
+        self.loaders = [SphinxFileSystemLoader(x) for x in loaderchain]
 
         use_i18n = builder.app.translator is not None
         extensions = use_i18n and ['jinja2.ext.i18n'] or []
