@@ -5,7 +5,7 @@
 
     Theming support for HTML builders.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,9 +13,11 @@ import os
 import shutil
 import zipfile
 import tempfile
-import ConfigParser
 from os import path
 
+from six import string_types, iteritems
+from six.moves import configparser
+
 try:
     import pkg_resources
 except ImportError:
@@ -24,10 +26,10 @@ except ImportError:
 from sphinx import package_dir
 from sphinx.errors import ThemeError
 
-
 NODEFAULT = object()
 THEMECONF = 'theme.conf'
 
+
 class Theme(object):
     """
     Represents the theme chosen in the configuration.
@@ -65,23 +67,56 @@ class Theme(object):
                 cls.themes[tname] = (path.join(themedir, theme), tinfo)
 
     @classmethod
-    def load_extra_themes(cls):
-        for themedir in load_theme_plugins():
-            if not path.isdir(themedir):
-                continue
-            for theme in os.listdir(themedir):
-                if not path.isfile(path.join(themedir, theme, THEMECONF)):
-                    continue
-                cls.themes[theme] = (path.join(themedir, theme), None)
+    def load_extra_theme(cls, name):
+        themes = ['alabaster']
+        try:
+            import sphinx_rtd_theme
+            themes.append('sphinx_rtd_theme')
+        except ImportError:
+            pass
+        if name in themes:
+            if name == 'alabaster':
+                import alabaster
+                themedir = alabaster.get_path()
+                # alabaster theme also requires 'alabaster' extension, it will be loaded
+                # at sphinx.application module.
+            elif name == 'sphinx_rtd_theme':
+                themedir = sphinx_rtd_theme.get_html_theme_path()
+            else:
+                raise NotImplementedError('Programming Error')
 
-    def __init__(self, name):
+        else:
+            for themedir in load_theme_plugins():
+                if path.isfile(path.join(themedir, name, THEMECONF)):
+                    break
+            else:
+                # specified theme is not found
+                return
+
+        cls.themepath.append(themedir)
+        cls.themes[name] = (path.join(themedir, name), None)
+        return
+
+    def __init__(self, name, warn=None):
         if name not in self.themes:
-            self.load_extra_themes()
+            self.load_extra_theme(name)
             if name not in self.themes:
-                raise ThemeError('no theme named %r found '
-                                 '(missing theme.conf?)' % name)
+                if name == 'sphinx_rtd_theme':
+                    raise ThemeError('sphinx_rtd_theme is no longer a hard dependency '
+                                     'since version 1.4.0. Please install it manually.')
+                else:
+                    raise ThemeError('no theme named %r found '
+                                     '(missing theme.conf?)' % name)
         self.name = name
 
+        # Do not warn yet -- to be compatible with old Sphinxes, people *have*
+        # to use "default".
+        # if name == 'default' and warn:
+        #     warn("'default' html theme has been renamed to 'classic'. "
+        #          "Please change your html_theme setting either to "
+        #          "the new 'alabaster' default theme, or to 'classic' "
+        #          "to keep using the old default.")
+
         tdir, tinfo = self.themes[name]
         if tinfo is None:
             # already a directory, do nothing
@@ -92,7 +127,8 @@ class Theme(object):
             self.themedir = tempfile.mkdtemp('sxt')
             self.themedir_created = True
             for name in tinfo.namelist():
-                if name.endswith('/'): continue
+                if name.endswith('/'):
+                    continue
                 dirname = path.dirname(name)
                 if not path.isdir(path.join(self.themedir, dirname)):
                     os.makedirs(path.join(self.themedir, dirname))
@@ -100,20 +136,24 @@ class Theme(object):
                 fp.write(tinfo.read(name))
                 fp.close()
 
-        self.themeconf = ConfigParser.RawConfigParser()
+        self.themeconf = configparser.RawConfigParser()
         self.themeconf.read(path.join(self.themedir, THEMECONF))
 
         try:
             inherit = self.themeconf.get('theme', 'inherit')
-        except ConfigParser.NoOptionError:
+        except configparser.NoOptionError:
             raise ThemeError('theme %r doesn\'t have "inherit" setting' % name)
+
+        # load inherited theme automatically #1794, #1884, #1885
+        self.load_extra_theme(inherit)
+
         if inherit == 'none':
             self.base = None
         elif inherit not in self.themes:
             raise ThemeError('no theme named %r found, inherited by %r' %
                              (inherit, name))
         else:
-            self.base = Theme(inherit)
+            self.base = Theme(inherit, warn=warn)
 
     def get_confstr(self, section, name, default=NODEFAULT):
         """Return the value for a theme configuration setting, searching the
@@ -121,7 +161,7 @@ class Theme(object):
         """
         try:
             return self.themeconf.get(section, name)
-        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+        except (configparser.NoOptionError, configparser.NoSectionError):
             if self.base is not None:
                 return self.base.get_confstr(section, name, default)
             if default is NODEFAULT:
@@ -141,9 +181,9 @@ class Theme(object):
         for conf in reversed(chain):
             try:
                 options.update(conf.items('options'))
-            except ConfigParser.NoSectionError:
+            except configparser.NoSectionError:
                 pass
-        for option, value in overrides.iteritems():
+        for option, value in iteritems(overrides):
             if option not in options:
                 raise ThemeError('unsupported theme option %r given' % option)
             options[option] = value
@@ -185,10 +225,10 @@ def load_theme_plugins():
         func_or_path = plugin.load()
         try:
             path = func_or_path()
-        except:
+        except Exception:
             path = func_or_path
 
-        if isinstance(path, basestring):
+        if isinstance(path, string_types):
             theme_paths.append(path)
         else:
             raise ThemeError('Plugin %r does not response correctly.' %
